variable "authorized_keys" {
  type    = string
  default = ""
}

variable "flash_device_path" {
  type    = string
  default = "/dev/sda"
}

variable "hostname" {
  type    = string
  default = "HashiPi0"
}

variable "img_name" {
  type    = string
  default = "raspi.img"
}

variable "img_size" {
  type    = string
  default = "4G"
}

variable "img_url" {
  type    = string
  default = "https://downloads.raspberrypi.org/raspios_lite_arm64/images/raspios_lite_arm64-2024-03-15/2024-03-15-raspios-bookworm-arm64-lite.img.xz"
}

variable "nomad_tls_ca" {
  type    = string
  default = "./tls/nomad/nomad-agent-ca.pem"
}
variable "nomad_tls_ca_p12" {
  type    = string
  default = "./tls/nomad/nomad-agent-ca.p12"
}

variable "nomad_tls_certs" {
  type    = string
  default = "./tls/nomad/certs/"
}

variable "nomad_encrypt" {
  type      = string
  default   = ""
  sensitive = true
}

variable "nomad_client" {
  type    = string
  default = "true"
}

variable "nomad_jenkins_gid" {
  type    = string
  default = "1312"
}

variable "nomad_jenkins_uid" {
  type    = string
  default = "1312"
}

variable "nomad_nfs_mount" {
  type    = string
  default = ""
}

variable "nomad_nfs_server" {
  type    = string
  default = ""
}

variable "nomad_nfs_target" {
  type    = string
  default = ""
}

variable "nomad_podman_driver_version" {
  type    = string
  default = "0.5.2"
}

variable "nomad_server" {
  type    = string
  default = "true"
}

variable "nomad_version" {
  type    = string
  default = "1.8.2"
}

variable "username" {
  type    = string
  default = "in0rdr"
}

variable "vault_addr" {
  type    = string
  default = "https://vault.in0rdr.ch"
}

variable "vault_tls_ca_cert" {
  type    = string
  default = "./tls/vault/ca/vault_ca.pem"
}

variable "vault_tls_ca_key" {
  type    = string
  default = "./tls/vault/ca/vault_ca.key"
}

variable "vault_tls_subj_alt_name" {
  type    = string
  default = "IP:127.0.0.1"
}

variable "vault_transit_server" {
  type    = string
  default = ""
}

variable "vault_transit_token" {
  type      = string
  default   = ""
  sensitive = true
}

variable "bao_version" {
  type    = string
  default = "2.0.0-beta20240618"
}

source "arm" "hashipi" {
  file_checksum_type    = "sha256"
  file_checksum_url     = "${var.img_url}.sha256"
  file_target_extension = "xz"
  file_unarchive_cmd    = ["xz", "-d", "$ARCHIVE_PATH"]
  file_urls             = ["${var.img_url}"]
  image_build_method    = "resize"
  image_chroot_env      = ["PATH=/usr/local/bin:/usr/local/sbin:/usr/bin:/usr/sbin:/bin:/sbin"]
  image_partitions {
    filesystem   = "vfat"
    mountpoint   = "/boot"
    name         = "boot"
    size         = "256M"
    start_sector = "8192"
    type         = "c"
  }
  image_partitions {
    filesystem   = "ext4"
    mountpoint   = "/"
    name         = "root"
    size         = "0"
    start_sector = "532480"
    type         = "83"
  }
  image_path                   = "${var.img_name}"
  image_size                   = "${var.img_size}"
  image_type                   = "dos"
  qemu_binary_destination_path = "/usr/bin/qemu-aarch64-static"
  qemu_binary_source_path      = "/usr/bin/qemu-aarch64-static"
}

build {
  sources = ["source.arm.hashipi"]

  provisioner "file" {
    destination = "/tmp/resizerootfs"
    source      = "arm-builder/scripts/resizerootfs/resizerootfs"
  }

  provisioner "file" {
    destination = "/tmp/resizerootfs.service"
    source      = "arm-builder/scripts/resizerootfs/resizerootfs.service"
  }

  provisioner "shell" {
    script = "bootstrap.sh"
    environment_vars = [
      "HOSTNAME=${var.hostname}",
      "USERNAME=${var.username}",
      "AUTHORIZED_KEYS=${var.authorized_keys}",
      "NOMAD_VERSION=${var.nomad_version}",
      "BAO_VERSION=${var.bao_version}"
    ]
  }

  provisioner "shell" {
    inline = ["mkdir /tmp/tls"]
  }

  provisioner "file" {
    destination = "/tmp/tls/nomad-agent-ca.pem"
    source      = "${var.nomad_tls_ca}"
  }
  provisioner "file" {
    destination = "/tmp/tls/nomad-agent-ca.p12"
    source      = "${var.nomad_tls_ca_p12}"
  }

  provisioner "file" {
    destination = "/tmp/tls/"
    source      = "${var.nomad_tls_certs}"
  }

  provisioner "shell" {
    script        = "nomad.sh"
    remote_folder = "/home/${var.username}"
    environment_vars = [
      "USERNAME=${var.username}",
      "NFS_SERVER=${var.nomad_nfs_server}",
      "NFS_MOUNT=${var.nomad_nfs_mount}",
      "NFS_MOUNT_TARGET=${var.nomad_nfs_target}",
      "NOMAD_ENCRYPT=${var.nomad_encrypt}",
      "NOMAD_SERVER=${var.nomad_server}",
      "NOMAD_CLIENT=${var.nomad_client}",
      "NOMAD_PODMAN_DRIVER_VERSION=${var.nomad_podman_driver_version}",
      "NOMAD_JENKINS_UID=${var.nomad_jenkins_uid}",
      "NOMAD_JENKINS_GID=${var.nomad_jenkins_gid}",
      "VAULT_ADDR=${var.vault_addr}"
    ]
  }

  provisioner "file" {
    destination = "/tmp/vault_ca.pem"
    source      = "${var.vault_tls_ca_cert}"
  }

  provisioner "file" {
    destination = "/tmp/vault_ca.key"
    source      = "${var.vault_tls_ca_key}"
  }

  provisioner "shell" {
    script        = "openbao.sh"
    remote_folder = "/home/${var.username}"
    environment_vars = [
      "USERNAME=${var.username}",
      "HOSTNAME=${var.hostname}",
      "NOMAD_SERVER=${var.nomad_server}",
      "VAULT_TLS_CA_CERT=/tmp/vault_ca.pem",
      "VAULT_TLS_CA_KEY=/tmp/vault_ca.key",
      "VAULT_TLS_SUBJ_ALT_NAME=${var.vault_tls_subj_alt_name}",
      "VAULT_TRANSIT_SERVER=${var.vault_transit_server}",
      "VAULT_TRANSIT_TOKEN=${var.vault_transit_token}"
    ]
  }
}
