#!/usr/bin/env bash
#
# Packer shell provisioner for HashiCorp Nomad on Raspberry Pi
# https://www.nomadproject.io/docs/install/production/deployment-guide

# set -o errexit
# set -o nounset
set -o xtrace

cd "/home/${USERNAME}"

# Move uploaded tls files
TLS_DIR=/etc/nomad.d/tls
mkdir -p $TLS_DIR
mv /tmp/tls/* $TLS_DIR

chmod 640 $TLS_DIR/*.pem
chmod 644 $TLS_DIR/dc1-{cli,client,server}*
chmod 644 $TLS_DIR/nomad-agent-ca.pem
chmod 644 $TLS_DIR/nomad-agent-ca.p12

# Install podman driver for Nomad
# https://developer.hashicorp.com/nomad/plugins/drivers/community/containerd
curl -LO "https://releases.hashicorp.com/nomad-driver-podman/${NOMAD_PODMAN_DRIVER_VERSION}/nomad-driver-podman_${NOMAD_PODMAN_DRIVER_VERSION}_linux_arm64.zip"
unzip "nomad-driver-podman_${NOMAD_PODMAN_DRIVER_VERSION}_linux_arm64.zip"
mkdir -p /opt/nomad/plugins/
mv nomad-driver-podman /opt/nomad/plugins/

# Create Nomad data directory
mkdir -p /opt/nomad

# Create Nomads config files
mkdir -p /etc/nomad.d
rm -rf /etc/nomad.d/*

cat << EOF > /etc/nomad.d/nomad.hcl
datacenter = "dc1"
data_dir = "/opt/nomad"

vault {
  # https://www.nomadproject.io/docs/configuration/vault
  # Nomad servers require a Vault Token set in /etc/nomad.d/nomad.env
  # Nomad clients use the use the allocation's Vault token
  enabled = true
  address = "$VAULT_ADDR"
  tls_skip_verify = true
$(if [[ "$NOMAD_SERVER" = true ]]; then
  # Only servers create further Tokens for clients from the Token role
  # https://developer.hashicorp.com/nomad/docs/configuration/vault#nomad-client
  echo -e "
  # default workload identity for bao
  # https://developer.hashicorp.com/nomad/docs/configuration/vault#default_identity
  default_identity {
    aud = ["vault.in0rdr.ch"]
    ttl = "1h"
  }"
fi)
}

telemetry {
  # https://developer.hashicorp.com/nomad/docs/configuration/telemetry
  publish_allocation_metrics = false
  publish_node_metrics       = true
}
EOF


# Configure as a Nomad server agent
#
# https://www.nomadproject.io/docs/configuration
# Note that it is strongly recommended not to operate a node as both client and server,
# although this is supported to simplify development and testing.
if [[ "$NOMAD_SERVER" = true ]]; then
cat << EOF > /etc/nomad.d/server.hcl 
server {
  enabled = true

  # Encrypt gossip communication
  encrypt = "$NOMAD_ENCRYPT"

  bootstrap_expect = 3
  server_join {
    retry_join = [ "pi0", "pi2", "pi4" ]
  }

  # https://developer.hashicorp.com/nomad/docs/configuration/server#raft_multiplier
  # https://developer.hashicorp.com/consul/docs/install/performance
  raft_multiplier = 5
  # https://developer.hashicorp.com/nomad/docs/configuration/server#client-heartbeats
  # Increase heartbeat ttl under unreliable network conditions to prevent
  # client: error heartbeating. retrying: error="failed to update status: rpc
  # error: Not ready to serve consistent reads"
  heartbeat_grace = "30s"
  min_heartbeat_ttl = "30s"
  default_scheduler_config {
    scheduler_algorithm             = "spread"
    memory_oversubscription_enabled = true
    reject_job_registration         = false
    pause_eval_broker               = false

    preemption_config {
      batch_scheduler_enabled    = true
      system_scheduler_enabled   = true
      service_scheduler_enabled  = true
      sysbatch_scheduler_enabled = true
    }
  }
}

tls {
  http = true
  rpc  = true

  ca_file   = "$TLS_DIR/nomad-agent-ca.pem"
  cert_file = "$TLS_DIR/dc1-server-nomad.pem"
  key_file  = "$TLS_DIR/dc1-server-nomad-key.pem"

  verify_server_hostname = true
  verify_https_client    = false
}
EOF
fi # endif NOMAD_SERVER

systemctl enable nomad

if [[ "$NOMAD_CLIENT" = true ]]; then
# this instance acts as a Nomad client agent
cat << EOF > /etc/nomad.d/client.hcl
client {
  enabled = true
  server_join {
    retry_join = [ "pi0", "pi2", "pi4" ]
  }

  # Allow jobs to use the loopback interface for exposing
  # private services only accessible from the client nodes
  host_network "private" {
    cidr = "127.0.0.1/32"
  }

  # Avoid resource contention between host/workload when oversubscribing
  # memory. Reserve a portion of the node's resources from receiving tasks.
  # https://developer.hashicorp.com/nomad/docs/job-specification/resources#memory-oversubscription
  reserved {
    # CPU to reserve, in MHz
    cpu = 500
    # memory to reserve, in MB
    memory = 256
  }
}

tls {
  http = true
  rpc  = true

  ca_file   = "$TLS_DIR/nomad-agent-ca.pem"
  cert_file = "$TLS_DIR/dc1-client-nomad.pem"
  key_file  = "$TLS_DIR/dc1-client-nomad-key.pem"

  verify_server_hostname = true
  verify_https_client    = false
}

# Enable raw exec driver (jobs w/o isolation)
plugin "raw_exec" {
  config {
    enabled = true
  }
}
EOF

# Mount NFS for hostpath volumes
if [[ -n "$NFS_SERVER" ]]; then
  mkdir -p "$NFS_MOUNT_TARGET"
  # mount $NFS_SERVER:/$NFS_MOUNT $NFS_MOUNT_TARGET
  cat << EOF >> /etc/fstab
$NFS_SERVER:/$NFS_MOUNT $NFS_MOUNT_TARGET nfs defaults  0 0
EOF
fi # endif NFS_SERVER

# Enable memory cgroups for NFS CSI
# https://downey.io/blog/exploring-cgroups-raspberry-pi
sed -i 's/$/ cgroup_enable=memory/' /boot/cmdline.txt

# Ensure iptable modules for Netreap
cat << EOF > /etc/modules-load.d/iptables.conf
iptable_nat
iptable_mangle
iptable_raw
iptable_filter
ip6table_mangle
ip6table_raw
ip6table_filter
EOF

# Install CNI plugins
# - https://developer.hashicorp.com/nomad/docs/networking/cni
curl -L -o cni-plugins.tgz "https://github.com/containernetworking/plugins/releases/download/v1.5.0/cni-plugins-linux-arm64-v1.5.0.tgz"
mkdir -p /opt/cni/bin
tar -C /opt/cni/bin -xzf cni-plugins.tgz

# Prepare a Jenkins user for Jenkins agents and workload
# https://code.in0rdr.ch/nomad/file/docker/docker-jenkins-inbound-agent/README.html
groupadd -g $NOMAD_JENKINS_GID jenkins
useradd -m -s /bin/bash -u $NOMAD_JENKINS_UID -g $NOMAD_JENKINS_GID jenkins

# keep my podman.socket enabled even if no jenkins user is logged in
loginctl enable-linger jenkins

# https://www.freedesktop.org/software/systemd/man/latest/systemctl.html#-M
systemctl --user -M jenkins@ start podman.socket
systemctl --user -M jenkins@ enable podman.socket

# create the mountpoint for the workspaces, podman does not create it for us
sudo -u jenkins mkdir /home/jenkins/workspace

# move Nomad server truststore
sudo mv $TLS_DIR/nomad-agent-ca.p12 /home/jenkins/
sudo chown jenkins: /home/jenkins/nomad-agent-ca.p12

fi # endif NOMAD_CLIENT

# Configure .bashrc
cat << EOF >> "/home/${USERNAME}/.bashrc"

complete -C /usr/bin/nomad nomad
export NOMAD_ADDR=https://127.0.0.1:4646
export NOMAD_CACERT=$TLS_DIR/nomad-agent-ca.pem
export NOMAD_CLIENT_CERT=$TLS_DIR/dc1-cli-nomad.pem
export NOMAD_CLIENT_KEY=$TLS_DIR/dc1-cli-nomad-key.pem
EOF
