#!/usr/bin/env bash
#
# Packer shell provisioner for OpenBao on Raspberry Pi
# https://openbao.org/docs/install

# set -o errexit
# set -o nounset
set -o xtrace

cd "/home/${USERNAME}"

# Create bao config directories
mkdir -p /etc/openbao
rm -rf /etc/openbao/*
mkdir -p /etc/openbao/tls
# The bao systemd service requires this env file, can be empty
touch /etc/openbao/openbao.env
cd /etc/openbao/tls

# Specify CSR parameters for server key
VAULT_TLS_SUBJ_ALT_NAME=${VAULT_TLS_SUBJ_ALT_NAME:+", $VAULT_TLS_SUBJ_ALT_NAME"}
SERVER_CONFIG="
[ req ]
commonName         = $HOSTNAME
distinguished_name = dn
req_extensions     = ext
[ dn ]
CN                 = Common Name
[ ext ]
subjectAltName     = DNS:$HOSTNAME $VAULT_TLS_SUBJ_ALT_NAME
keyUsage=critical,digitalSignature,keyAgreement
"
# Create new private key and CSR
openssl req -config <(echo "$SERVER_CONFIG") -subj "/CN=${HOSTNAME}" -extensions ext -out "${HOSTNAME}.csr" -new -newkey rsa:2048 -nodes -keyout "${HOSTNAME}.key"
# Sign the CSR
openssl x509 -extfile <(echo "$SERVER_CONFIG") -extensions ext -req -in "${HOSTNAME}.csr" -CA "$VAULT_TLS_CA_CERT" -CAkey "$VAULT_TLS_CA_KEY" -CAcreateserial -out "${HOSTNAME}.pem" -days 365
# Show fingerprint
openssl x509 -in "${HOSTNAME}.pem" -fingerprint -noout

# Cleanup CA key
rm -rf "$VAULT_TLS_CA_KEY"

# Change permissions for tls certs
chmod 640 *.key
chmod 644 *.pem

# Concatenate CA and server certificate
cat "$VAULT_TLS_CA_CERT" >> "${HOSTNAME}.pem"

# Trust the CA
mv "$VAULT_TLS_CA_CERT" /usr/local/share/ca-certificates/
update-ca-certificates

cat << EOF > /etc/openbao/openbao.hcl
ui = true

listener "tcp" {
  address       = "0.0.0.0:8200"
  tls_cert_file = "/etc/openbao/tls/$HOSTNAME.pem"
  tls_key_file  = "/etc/openbao/tls/$HOSTNAME.key"
  tls_disable_client_certs = true
}

# HA advertisement addresses
#
# https://openbao.org/docs/configuration/#high-availability-parameters
# https://openbao.org/docs/concepts/ha#client-redirection
#
# API_ADDR for client redirection (fallback, if request forwarding is
# disabled). Uses go-sockaddr template to fetch the actual ip for an interface
api_addr = "https://{{GetPrivateInterfaces | exclude \"type\" \"IPv6\" | include \"name\" \"eth0\" | attr \"address\" }}:8200"
# CLUSTER_ADDR: Vault listens for server-to-server cluster requests
cluster_addr = "https://{{GetPrivateInterfaces | exclude \"type\" \"IPv6\" | include \"name\" \"eth0\" | attr \"address\" }}:8201"

storage "raft" {
  path = "/opt/openbao/data"
  node_id = "$HOSTNAME"

  retry_join {
    leader_api_addr = "https://pi0:8200"
    leader_tls_servername = "vault.in0rdr.ch"
    leader_ca_cert_file = "/etc/openbao/tls/$HOSTNAME.pem"
  } 
  retry_join {
    leader_api_addr = "https://pi2:8200"
    leader_tls_servername = "vault.in0rdr.ch"
    leader_ca_cert_file = "/etc/openbao/tls/$HOSTNAME.pem"
  } 
  retry_join {
    leader_api_addr = "https://pi4:8200"
    leader_tls_servername = "vault.in0rdr.ch"
    leader_ca_cert_file = "/etc/openbao/tls/$HOSTNAME.pem"
  } 
}

seal "transit" {
  address = "$VAULT_TRANSIT_SERVER"
  disable_renewal = "false"
  key_name = "autounseal"
  mount_path = "transit/"
  tls_skip_verify = "true"
}
EOF

echo "VAULT_TOKEN=$VAULT_TRANSIT_TOKEN" > /etc/openbao/openbao.env

chmod 640 /etc/openbao/openbao.hcl

# only enable openbao on the Nomad servers
if [[ "$NOMAD_SERVER" = true ]]; then
 systemctl enable openbao
fi

# Configure .bashrc
cat << EOF >> "/home/${USERNAME}/.bashrc"

complete -C /usr/bin/bao boa
export VAULT_ADDR="https://$HOSTNAME:8200"
export VAULT_SKIP_VERIFY=true
EOF

# Change ownership for config directory 
chown -R openbao: /etc/openbao/

echo 0
